
 /*  botmon.c   June 20 2001  by Dale Heatherington */

/* To compile: gcc -o botmon botmon.c -lslang */

#include <stdio.h>
#include <termio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <slang/slang.h>

#define BAUDRATE B9600

#define bit08 unsigned char
#define bit16 unsigned short int
#define SYNC0  0xff
#define SYNC1  0xf0
#define SYNC2  0x55






#define MODE_STOP 0
#define MODE_TEST 0xff







#define FWD 1
#define FWD_LEFT 2
#define FWD_RIGHT 3
#define FWD_HARD_LEFT 4
#define FWD_HARD_RIGHT 5
#define FWD_MAX 6
   
#define SPIN_LEFT 10
#define SPIN_RIGHT 11
#define STOP 12

#define REV 16
#define REV_LEFT 17
#define REV_RIGHT 18
#define REV_HARD_LEFT 19
#define REV_HARD_RIGHT 20
#define REV_MAX 21

#define comport "/dev/botmon"

static bit08 get_byte();
static void get_bytes(bit08 *, bit16);
static bit16 get_word();
static void get_data(bit16 *, bit16);

static void send_byte(bit08);
static void send_word(bit16);
static void send_data(bit16 *, bit16);

static void timed_read(bit08 *, bit16);
static int open_tty(char *name) ;


int tty;

FILE *ftty;

struct packet {
   bit08 address;
   bit08 count;
   bit08 data[40];
};

union raw_packet {
   struct   packet pkt;
   bit08    raw[41];
};

union raw_packet  rxdata,txdata;




/*---------------------------------------------------------------------*/
print_hex(bit08 *cp, int size)
{  int i;
   
   for(i=0;i<size;i++){
      SLsmg_printf("%02X ",cp[i]);
   }
   SLsmg_printf("\n");
}


fprint_hex(FILE* f, bit08 *cp, int size){
   int i;
   for(i=0;i<size;i++){
      fprintf(f,"%02X ",cp[i]);
   }
   fprintf(f,"\n");
}



print_hex_word(bit16 *wp, int size){
   int i;
  
  for(i=0;i<size;i++){
     SLsmg_printf("%04X ",wp[i]);
  }
  SLsmg_printf("\n");
}


print_binary_byte(bit08 cp){
   int i;
   char *sb = (char*)malloc(9);
   
   for(i=0;i<8;i++){
      if(cp & 0x80) sb[i] = '1'; else sb[i] = '0';
      cp = cp << 1;
   }

   SLsmg_printf("%s ",sb);
   free(sb);
   
}

   

/*---------------------------------------------------------------------*/

static int 
open_tty(char *name)
{
   struct termios termios;


   tty = open(name, O_RDWR, 0777);
   if (tty == -1) {
      perror("open of com port failed");

      exit(1);
   }

   if (tcgetattr(tty, &termios) == -1) {
      perror("getting attributes of programmer port failed");
      exit(1);
   }

 
   termios.c_iflag = IGNBRK  ;//| IGNPAR;
   termios.c_oflag = 0;
   termios.c_cflag = BAUDRATE | CS8 | CREAD | CLOCAL;
   termios.c_lflag = 0;
   termios.c_cc[VMIN] = 0; /* Timed read */
   termios.c_cc[VTIME] = 1; /* 0.1 second timeout  */
   if (tcsetattr(tty, TCSANOW, &termios) == -1) {
      perror("setting attributes of com port failed");
      exit(1);
   }

   cfmakeraw(&termios);
}

/*-----------------------------------------------*/
static void
reopen_tty(char* name){

   close(tty);
   open_tty(comport);
}

/*----------------------------------------*/
static void
send_byte(bit08 byte)
{
   int modem_bits = 0;
   bit08 bbuf[1];

   bbuf[0] = byte;
   write(tty, bbuf, 1);
   //log_write(byte);
   ioctl(tty, TCSBRK, 1); /* Wait until the characters have been output */
   //usleep(WAIT);          /* wait a fixed time - needs this for some reason */

}

/*-----------------------------------------*/
static bit08
get_byte()
{
   bit08 buf[1];

   timed_read(buf, 1);
   return buf[0];
}

/*---------------------------------------------*/

static void
get_bytes(bit08 *buffer, bit16 length)
{
   timed_read(buffer, length);

}

/*---------------------------------------------------*/


static void
get_data(bit16 *buffer, bit16 length)
{ unsigned int i,j,k;
   timed_read((bit08 *)buffer, length * 2);

   for (i=0;i<length;i++) {
      buffer[i] = ntohs(buffer[i]);  //Exchange hi and lo bytes for Intel cpu byte order
   }
}

/*----------------------------------------------------*/
static void
timed_read(bit08 *buffer, bit16 length)
{
   int total = length;
   int count;
   int i;

   while (length) {
      count = read(tty, (char *)buffer, length);
      if (count <= 0) {
         fprintf(stderr,
                 "read failed: wanted %d got %d error %d\n",
                 total, total - length, count);
         exit(3);
      }
      for (i=0; i < count; i++) {

         //log_read(buffer[i]);
      }


      length -= count;
      buffer += count;
   }
}
/*-------------------------------------------------------*/
void crc(bit08 data, bit16 *crc16)

#define MSB 0x8000
#define  MASK 0x8005
{  int k;
   bit16 c,d,t ;
      
      c = data << 8 ;
      d = c;
      for (k=0;k<8;k++) {
         *crc16 = (c & MSB) ^ *crc16;

         if (*crc16 & MSB) {
            *crc16 = *crc16 << 1;
            *crc16 = *crc16 ^ MASK;
         } else 
            *crc16 = *crc16 << 1;
         d = d << 1;
         c = d;
      }
  
}

/*-------------------------------------------------------*/
void
send_pkt(union raw_packet *data)
{  int i,j;
   int pktsize;
   bit08 c,c1,c2;
   bit16 crc16 = 0;

   c = SYNC0;
   for(i=0;i<8;i++) write(tty,&c,1); //leading rubouts

   c = SYNC1;
   write(tty,&c,1);         //Send sync 1
   c = SYNC2;
   write(tty,&c,1);         //Send sync 2

   j = data->pkt.count +2;  //Add two for count and address 
   
   for (i=0;i<j;i++) {      //Send address,count and data
      crc(data->raw[i],&crc16);
      write(tty,&data->raw[i],1);
   }

   c1 = crc16 >> 8;         //send crc H
   write(tty,&c1,1);
   c2 = crc16 & 0xff;       //send crc L
   write(tty,&c2,1);

      //debug
   /*
   pktsize = data->pkt.count + 1;
   data->raw[pktsize++] = c1;
   data->raw[pktsize++] = c2;
   SLsmg_gotorc(0,0);
   print_hex(data->raw,pktsize);
   crc16 = 0;
   for(i=0;i<pktsize;i++){
      crc(data->raw[i],&crc16);
      
   }

   SLsmg_gotorc(1,0);
   SLsmg_write_string("CRC= ");
   print_hex_word(&crc16,1);
   //sleep(10);
   */

}

/*------------------------------------------------------*/
int
recv_pkt(union raw_packet *data)
{  int i,j,count,pktsize;
   bit08 sync,a;
   bit16 crc16 = 0;
   static int x = 0;

   
   count = 1;
   while((count == 1) && (sync != SYNC1)){
   count = read(tty,&sync, 1);
   if(count < 1) reopen_tty(comport);   //kludge to work around com port lockups on random data
     }
   if((count <= 0) || (sync != SYNC1)) return 1;

     count = read(tty,&sync,1);

     if((count <= 0) || (sync != SYNC2)) return 1;
   count = read(tty,&data->pkt.address,2); // get address and count
  
   if((count <= 0) || (data->pkt.count > 40)) return 2;
   pktsize = data->pkt.count + 2; //add 2 for crc bytes
   i = 2;
   j = pktsize + 2;
   while((count > 0) && (i < j)){
      count = read(tty,&data->raw[i],1);
      i++;
      

   }

     if(i != j) return 3;
   
   for(i=0;i<j;i++){
      crc(data->raw[i],&crc16);
       }
  
   if(crc16 != 0) return 4;

   return 0;
}  

/*------------------------------------------------------*/


      
/* 
      tx_buf[i++] = 0x01;        //data type = 1
      tx_buf[i++] = l_speed;  //1
      tx_buf[i++] = r_speed; //2
      tx_buf[i++] = vac_nav_state;        //3
      tx_buf[i++] = runTimer_H;  //4
      tx_buf[i++] = runTimer_L;  //5

      tx_buf[i++] = drive_battery; //6
      tx_buf[i++] = 0 ;             //7
      tx_buf[i++] = speed;          //8 
      tx_buf[i++] = bot_mode;      //9
      
      tx_buf[i++] = delta_left_wall;          //10
      tx_buf[i++] = delta_right_wall;          //11
      tx_buf[i++] = timer;         //12
      tx_buf[i++] = odometer >> 8;      //13  odometer high byte
      tx_buf[i++] = odometer & 0xff;    //14  odometer low byte
      tx_buf[i++] = drive_current; //15
      
      for(j=0;j<8;j++){
         tx_buf[i++] = spi_buf[j];  //16..24 are SPI port buffer data
*/
void
print_info(union raw_packet *pkt, int row, int col)
{
   char left_speed,right_speed,track,rmfb,lmfb;
   char line_find,speed,base_mode;
   
   unsigned char drive_battery,bot_mode,timer,drive_current;
   unsigned char a_sonar_rf, a_sonar_lf, d_sonar_rf, d_sonar_lf ;
   unsigned char a_sonar_rr, a_sonar_lr, d_sonar_rr, d_sonar_lr,  pot;
   unsigned char tilt_dir,lr_ratio;
   unsigned char lf_rr_sens, rf_lr_sens;
   char arb_code;
   unsigned char portb, c, portc, line_cor, dir_ctrl, flags;
   unsigned int runTimer;
   char* vns;
   float ips;
   float drive_battery_voltage;
   float vacuum_battery_voltage;
   float drive_current_amps;
   
     
   char *mode;
  
   char *cpDir = NULL;
   char *cpTilt = NULL;

   left_speed = pkt->pkt.data[1];
   right_speed = pkt->pkt.data[2];
   
   flags = pkt->pkt.data[3];
   runTimer = (pkt->pkt.data[4] * 256) + pkt->pkt.data[5];
   lr_ratio = pkt->pkt.data[6];
   drive_battery = pkt->pkt.data[10];
   
   tilt_dir = pkt->pkt.data[8];
   bot_mode = pkt->pkt.data[9];
   
   dir_ctrl = pkt->pkt.data[11];
   timer = pkt->pkt.data[12];
   lf_rr_sens = pkt->pkt.data[13];
   rf_lr_sens = pkt->pkt.data[14];

   drive_current = pkt->pkt.data[15];
   arb_code = pkt->pkt.data[16];
   
   
   d_sonar_lf = pkt->pkt.data[20];
   a_sonar_lf = pkt->pkt.data[21];
   d_sonar_rf = pkt->pkt.data[22];
   a_sonar_rf = pkt->pkt.data[23];
   d_sonar_lr = pkt->pkt.data[24];
   a_sonar_lr = pkt->pkt.data[25];
   d_sonar_rr = pkt->pkt.data[26];
   a_sonar_rr = pkt->pkt.data[27];
   pot = pkt->pkt.data[28];
      
   if(tilt_dir & 0x80) {
      cpDir = "REVERSE";
   }else cpDir = "FORWARD";

   if(tilt_dir & 0x40){
      cpTilt = "      ";
   }
      else cpTilt = "*TILT*";


   drive_battery_voltage = ((float)drive_battery/256) * 15;
   drive_current_amps    = ((float)drive_current/256) * 5;

   SLsmg_gotorc(row++,col);

   SLsmg_printf("LEFT SPD: %4d  RIGHT SPD: %4d  LF-RR: %4d  RF-LR: %4d ", left_speed,
                right_speed, lf_rr_sens, rf_lr_sens);
   SLsmg_gotorc(row++,col);
   SLsmg_printf("RUN TIMER:%5d  BATTERY:%4.1f  DRIVE AMPS:%3.2f",
                 runTimer,drive_battery_voltage,drive_current_amps);
   SLsmg_gotorc(row++,col);
   SLsmg_printf("MODE:%4d  TIMER:%4d  LR-Sonar-ratio:%4d  %s  %s", bot_mode,timer,lr_ratio,cpDir,cpTilt);

   SLsmg_gotorc(row++,col);
   
   SLsmg_printf("Sonar: Left Front: %4d %4d  Right Front: %4d %4d  ",d_sonar_lf,a_sonar_lf,d_sonar_rf, a_sonar_rf);
   
   SLsmg_gotorc(row++,col);
   SLsmg_printf("Sonar: Left Rear : %4d %4d  Right Rear : %4d %4d  ",d_sonar_lr,a_sonar_lr,d_sonar_rr, a_sonar_rr);
   SLsmg_gotorc(row++,col);
   SLsmg_printf("Pot  : %4d  ",pot);
   
   
   

  
   

   SLsmg_gotorc(row+2,col);
   switch(arb_code){
   case  0: mode = "NULL       ";
            break;
   case 1:  mode = "CRUSE      ";
            break;
  case 2:
            mode = "CHASE      ";
            break;

   case 3: mode =  "TILT       ";
            break;
   case 4:  mode = "PUSHED BACK";
            break;
   case 5:  mode = "BORDER";
            break;
   case 6:  mode = "CURRENT LIM";
            break;
   case 7:  mode = "LOW VOLTAGE";
            break;
            
   default: mode = "Arbitration code unknown";
            break;
   }

   SLsmg_gotorc(row++,col);
   SLsmg_printf("Arbitator Output: %s", mode);

   switch(dir_ctrl){
   case STOP:           vns = "Stopped           " ; break;
   case FWD:            vns = "Forward           " ; break;
   case FWD_MAX:        vns = "Forward MAX       " ; break;
   case FWD_LEFT:       vns = "Forward LEFT      " ; break;
   case FWD_HARD_LEFT:  vns = "Forward HARD LEFT "; break;
   case FWD_RIGHT:      vns = "Forward RIGHT     "; break;
   case FWD_HARD_RIGHT: vns = "Forward HARD RIGHT" ; break;
   case REV:            vns = "Reverse           " ; break;
   case REV_MAX:        vns = "Reverse MAX       " ; break;
   case REV_LEFT:       vns = "Reverse LEFT      "; break;
   case REV_HARD_LEFT:  vns = "Reverse HARD LEFT " ; break;
   case REV_RIGHT:      vns = "Reverse RIGHT     "; break;
   case REV_HARD_RIGHT: vns = "Reverse HARD RIGHT"; break;
   case SPIN_LEFT:      vns = "Spin Left         "; break;
   case SPIN_RIGHT:     vns = "Spin Right        "; break;
   default:             vns = "Unknown           "; break;

   }

  
   SLsmg_printf("                                     "); //erase line
   SLsmg_gotorc(row++,col);                                 //print bot mode
   SLsmg_printf("Dir Ctl: %s ",vns);
   
   SLsmg_gotorc(row++,col);
          

   vns = "         ";
   if(flags & (1 << 6)) vns = "ATTACK";
   SLsmg_printf("FLAGS: %s ",vns);

  

}

/*------------------------------------------------------*/
void looptest()
{
   int rc,i;
   bit08 type;
   char ch;
   unsigned int uch;
   int ctr;
   FILE* f;
   //strcpy(txdata.pkt.data,"Data test");
   //txdata.pkt.count = strlen(txdata.pkt.data);
   //txdata.pkt.address = 1;
   //send_pkt(&txdata);
   
   // SLsmg_gotorc(1,1);
  //  SLsmg_printf("In looptest");
  //  SLsmg_refresh();

   ctr = 0;

   f = fopen("botdata.log", "w");

     if (f == NULL) {
        fprintf(stderr,"failed...cannot create file botdata.log\n");
        exit(1);
     }

   i = 1;
   while(1 ==1){
     
      if((rc = recv_pkt(&rxdata)) == 0) {
         SLsmg_gotorc(2,0);
         print_hex(rxdata.raw,rxdata.pkt.count+2); //Print raw packet in hex
         fprint_hex(f,rxdata.raw,rxdata.pkt.count+2); //Print raw packet in hex to file
         type = rxdata.pkt.data[0];
         if(type == 1){
            
            print_info(&rxdata,4,0);
         } else
            { printf("CRC ERROR");
         }
         
         
        SLsmg_gotorc(0,0);
        SLsmg_refresh();
      }
     
      //ctr++;
      //SLsmg_gotorc(0,0);
      //print_hex(rxdata.raw,5);
      //SLsmg_refresh();

      if(SLang_input_pending(0)){
         ch = SLang_getkey();
         if(ch == 'q') {
            fclose(f);
            return;
         }
      }

   }
         

}

/*------------------------------------------------------*/

int main(int argc, char *argv[])
{  char ch = '\0';
   bit08 b;


   open_tty(comport);
   

   SLang_init_tty(-1,0,1);
   SLtt_get_terminfo();
   SLsmg_init_smg();

   SLsmg_gotorc(0,20);
   //SLtt_set_color(0,NULL,"white","blue");
   SLsmg_write_string("RoboR MiniSumo Robot Data Link");
   
   
   SLsmg_refresh();
   looptest();

   
   while (ch != 'q') {
      ch = SLang_getkey();
      printf("read: %c 0x%x\n", isprint(ch) ? ch : ' ', ch);
   }
    
   SLsmg_reset_smg();
   SLang_reset_tty();


   exit(0);
}



